<?php

namespace TDevAgency\CmsSitemap\Providers;

use DaveJamesMiller\Breadcrumbs\Facades\Breadcrumbs;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider;
use TDevAgency\Cms\Commands\AddUser;
use TDevAgency\Cms\Commands\CmsInstall;
use TDevAgency\Cms\Commands\PublicAssets;
use TDevAgency\Cms\Models\BaseModels\AdminUser;
use TDevAgency\Cms\Observers\AdminUserObserver;

class CmsSitemapServiceProvider extends ServiceProvider
{


    /**
     * @var string
     */
    private $rootPath;

    public function __construct($app)
    {
        parent::__construct($app);

        $this->rootPath = __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR;
    }

    /**
     * Register services.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(
            $this->rootPath . 'config' . DIRECTORY_SEPARATOR . 'cms-sitemap.php', 'cms-sitemap'
        );
    }

    /**
     * Merge the given configuration with the existing configuration.
     *
     * @param string $path
     * @param string $key
     * @return void
     */
    protected function mergeConfigFrom($path, $key): void
    {
        $config = $this->app['config']->get($key, []);
        $this->app['config']->set($key, $this->mergeConfigs($config, require $path));
    }

    /**
     * @param array $original
     * @param array $merging
     * @return array
     */
    protected function mergeConfigs(array $original, array $merging): array
    {
        $array = array_merge($original, $merging);
        foreach ($original as $key => $value) {
            if (!is_array($value)) {
                continue;
            }
            if (!Arr::exists($merging, $key)) {
                continue;
            }
            if (is_numeric($key)) {
                continue;
            }
            $array[$key] = $this->mergeConfigs($value, $merging[$key]);
        }
        return $array;
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {

        AdminUser::observe(AdminUserObserver::class);

        $this->publishes([
            $this->rootPath . 'config' . DIRECTORY_SEPARATOR . 'cms-sitemap.php' => config_path('cms-sitemap.php'),
        ], 'config');
    }

}
